{
*****************************************************************************
*  Name    : USB Kaitenkei set RPM                                          *
*  Author  : Hiroyuki Iizuka                                                *
*  Date    : 2011/07/17                                                     *
*  Version : 1.0                                                            *
*  Notes   :                                                                *
*          :                                                                *
*****************************************************************************
}
Program kaitenkeiSetRPM

'---- configuration ---------------------------------------------------------
Device = 18F4550
Clock  = 48

Config
   USBDIV  = 1, 
   PLLDIV  = 5,
   CPUDIV  = OSC1_PLL2,
   FOSC    = HSPLL_HS,
   MCLRE   = OFF

'---- include file ----------------------------------------------------------
Include "Convert.bas"
Include "EEPROM.bas"

'---- EEPROM initialize -----------------------------------------------------
EEPROM(0) = (1000 As LongWord)

'---- interrupt -------------------------------------------------------------
#option org_reset     = $2000
#option vector_isr_hi = $2008
#option vector_isr_lo = $2018

Const ipLow  = 1
Const ipHigh = 2

Const High_priority = 1
Const Low_priority  = 0

Dim Interrupt_HIGH_enable As INTCON.booleans(7)
Dim Interrupt_LOW_enable  As INTCON.booleans(6)
Dim Interrupt_priority    As RCON.booleans(7)

'---- 7segment LED const definitions ----------------------------------------
Const Digit_N  = $00   ' Blank
Const Digit_0  = $3F
Const Digit_1  = $06
Const Digit_2  = $5B
Const Digit_3  = $4F
Const Digit_4  = $66
Const Digit_5  = $6D
Const Digit_6  = $7D
Const Digit_7  = $07
Const Digit_8  = $7F
Const Digit_9  = $6F
Const Digit_DP = $80

Const LED_ON  = 1
Const LED_OFF = 0

Dim LED_DEV6     As PORTC.0
Dim LED_DEV5     As PORTC.1
Dim LED_DEV4     As PORTC.2
Dim LED_DEV3     As PORTE.0
Dim LED_DEV2     As PORTE.1
Dim LED_DEV1     As PORTE.2

Dim LED_SEG_PORT As PORTD
Dim LED_SEG_A    As PORTD.0
Dim LED_SEG_B    As PORTD.1
Dim LED_SEG_C    As PORTD.2
Dim LED_SEG_D    As PORTD.3
Dim LED_SEG_E    As PORTD.4
Dim LED_SEG_F    As PORTD.5
Dim LED_SEG_G    As PORTD.6
Dim LED_SEG_DP   As PORTD.7

Dim led_val1       As Byte
Dim led_val2       As Byte
Dim led_val3       As Byte
Dim led_val4       As Byte
Dim led_val5       As Byte
Dim led_val6       As Byte
Dim led_pos        As Byte

Dim set_val(6)  As Byte

'------------ ROTARY ENCODER ------------------------------------------------
Dim ENCODER_A As PORTA.0
Dim ENCODER_B As PORTA.1

Const CW             =  1
Const CCW            = -1
Const ENCODER_STOP   =  0

Const ROTARY_LOCK    = 1
Const ROTARY_UNLOCK  = 0

Dim Rotary_Enc_A   As Byte
Dim Rotary_Enc_B   As Byte
Dim Rotary_A_Lock  As Boolean
Dim Rotary_B_Lock  As Boolean
Dim Rotary_Stat    As ShortInt
Dim turn           As ShortInt

'------------ Push Switch ---------------------------------------------------
Dim SW1 As PORTB.4
Dim SW2 As PORTA.3

Const SW_ON          = 0
Const SW_OFF         = 1

Const SW_ENABLE      = 0
Const SW_DISABLE     = 1

Dim SW1_status As Byte
Dim SW2_status As Byte

'------------ HALL sensor ---------------------------------------------------
Dim PHOTO_REF_PORT As PORTB.0

'------------ 3mm LED -------------------------------------------------------
Dim LED1 As PORTB.5
Dim LED2 As PORTB.3
Dim LED3 As PORTA.2

'------------ Mechanical RELAY ----------------------------------------------
Dim RELAY As PORTA.5

Const RELAY_ON       = 0
Const RELAY_OFF      = 1


'-------- INT0 --------------------------------------------------------------
Dim PortB0_changed         As INTCON.booleans(1)
Dim PortB0_int_enable      As INTCON.booleans(4)
Dim PortB0_int_rising_edge As INTCON2.booleans(6)

'-------- TMR0 --------------------------------------------------------------
Dim Timer0_overflowd             As INTCON.booleans(2)
Dim Timer0_interrupt_enable      As INTCON.booleans(5)
Dim Timer0_priority              As INTCON2.bits(2)
Dim Timer0_enable                As T0CON.booleans(7)
Dim Timer0_mode_8bit             As T0CON.booleans(6)
Dim Timer0_use_external_clock    As T0CON.booleans(5)
Dim Timer0_prescale_NOT_assign   As T0CON.booleans(3)
Dim Timer0_prescale_bit2         As T0CON.bits(2)
Dim Timer0_prescale_bit1         As T0CON.bits(1)
Dim Timer0_prescale_bit0         As T0CON.bits(0)

'-------- TMR1 --------------------------------------------------------------
Dim Timer1_overflowd             As PIR1.booleans(0) 
// Dim Timer1_peripheral_int_enable as INTCON.booleans(6)
Dim Timer1_interrupt_enable      As PIE1.booleans(0) 
Dim Timer1_prescale_bit0         As T1CON.bits(4)
Dim Timer1_prescale_bit1         As T1CON.bits(5)
Dim Timer1_use_external_clock    As T1CON.booleans(1)
Dim Timer1_enable                As T1CON.booleans(0)
Dim Timer1_priority              As IPR1.bits(0)

'-------- A/D converter -----------------------------------------------------
Dim ADconfig3 As ADCON1.3
Dim ADconfig2 As ADCON1.2
Dim ADconfig1 As ADCON1.1
Dim ADconfig0 As ADCON1.0

'-------- comparator --------------------------------------------------------
Dim Comparator_mode2 As CMCON.bits(2)
Dim Comparator_mode1 As CMCON.bits(1)
Dim Comparator_mode0 As CMCON.bits(0)

'-------- PORTB -------------------------------------------------------------
Dim PortB_pullups_disable As INTCON2.booleans(7)

'---- application [kaitenkei standard] --------------------------------------
Const CONST_MIN       = 720000000 ' TMR1 count 1 min @ 48MHz
Const ZERO_CLEAR_TIME = 1500      ' 1500 x 2ms = 3.0 sec
Const HALF_SECOND     = 91        ' timer1 overflow @ 0.5sec

Structure LWBstruc
  LongwordVal As LongWord Union  
  WordVal(2)  As Word     Union
  ByteVal(4)  As Byte     Union
End Structure 

Dim tempLWB As LWBstruc

Dim start_flag     As Boolean
Dim display_flag   As Boolean
Dim of_count       As Word     ' over flow counter

Dim TMR1L_value    As Byte
Dim TMR1H_value    As Byte
Dim TMR1_OVF_value As Word

Dim rpm            As LongWord
Dim tmr1_count     As LongWord
Dim count_div      As LongWord
Dim count_tmp1     As LongWord
Dim count_tmp2     As LongWord
Dim rotate         As LongWord
Dim rotate_value   As LongWord

Dim zero_display_counter As Word

Dim count_status   As Byte
Const DISPLAY = 0
Const SET     = 1

Dim set_rpm     As LongWord
Dim tmp_set_rpm As LongWord


'===============================================================================
' interrupt priority HIGH
'===============================================================================
Interrupt OnChangePORTB(ipHigh)
  '---- signal was detected
  If PortB0_changed Then
    If start_flag = true Then   ' save TMR1 value
      TMR1L_value = TMR1L
      TMR1H_value = TMR1H
      rotate = rotate + 1
      If of_count >= HALF_SECOND Then
        TMR1_OVF_value = of_count
        TMR1L = 0
        TMR1H = 0
        rotate_value = rotate
        of_count = 0
        rotate   = 0
        display_flag = true
      End If
    Else
      TMR1L = 0
      TMR1H = 0
      of_count = 0
      rotate   = 0
      start_flag = true
    End If
    zero_display_counter = 0
    PortB0_changed = false
  End If
End Interrupt


'===============================================================================
' interrupt priority LOW
'===============================================================================
Interrupt OnTimer(ipLow)

  '---- TMR1 overflow ----------
  If Timer1_overflowd Then
    Timer1_overflowd = false
    of_count = of_count + 1
  End If

  '--- TMR0 overflow (7SEG_LED storobe)   2 msec
  If Timer0_overflowd Then
    TMR0L = 69
    Select led_pos
      Case 1
        LED_DEV6 = LED_OFF
        LED_SEG_PORT = led_val1
        LED_DEV1 = LED_ON
        led_pos = led_pos + 1
      Case 2
        LED_DEV1 = LED_OFF
        LED_SEG_PORT = led_val2
        LED_DEV2 = LED_ON
        led_pos = led_pos + 1
      Case 3
        LED_DEV2 = LED_OFF
        LED_SEG_PORT = led_val3
        LED_DEV3 = LED_ON
        led_pos = led_pos + 1
      Case 4
        LED_DEV3 = LED_OFF
        LED_SEG_PORT = led_val4
        LED_DEV4 = LED_ON
        led_pos = led_pos + 1
      Case 5
        LED_DEV4 = LED_OFF
        LED_SEG_PORT = led_val5
        LED_DEV5 = LED_ON
        led_pos = led_pos + 1
      Case 6
        LED_DEV5 = LED_OFF
        LED_SEG_PORT = led_val6
        LED_DEV6 = LED_ON
        led_pos = 1
    EndSelect
    zero_display_counter = zero_display_counter + 1
    Timer0_overflowd = false   ' Clear interrupt flag
  End If
End Interrupt


'===============================================================================
' decode ascii-char -> 7segment-LED
'===============================================================================
Function led7seg_decode( ascii_char As Char ) As Byte
  Select ascii_char
    Case " "  result = Digit_N
    Case "0"  result = Digit_0
    Case "1"  result = Digit_1
    Case "2"  result = Digit_2
    Case "3"  result = Digit_3
    Case "4"  result = Digit_4
    Case "5"  result = Digit_5
    Case "6"  result = Digit_6
    Case "7"  result = Digit_7
    Case "8"  result = Digit_8
    Case "9"  result = Digit_9
    Else result = Digit_N
  EndSelect
End Function

'============================================================================
' Rotary encoder
'
' RETURN
'    CW / CCW / OTHER
'============================================================================
Function rotary_encoder() As ShortInt
  Dim temp_A As Byte
  Dim temp_B As Byte

  temp_A = ENCODER_A
  temp_B = ENCODER_B

  result = ENCODER_STOP

  If (Rotary_Enc_A = 1) And (temp_A = 0) And (temp_B = 1) Then
    Rotary_Stat   = CW
    Rotary_A_Lock = True
  End If
  If Rotary_A_Lock = True Then
    If (temp_A = 0) And (temp_B = 0) Then
      result = Rotary_Stat
      Rotary_A_Lock = False
    End If
  End If

  If (Rotary_Enc_B = 1) And (temp_B = 0) And (temp_A = 1) Then
    Rotary_Stat   = CCW
    Rotary_B_Lock = True
  End If
  If Rotary_B_Lock = True Then
    If (temp_A = 0) And (temp_B = 0) Then
      result = Rotary_Stat
      Rotary_B_Lock = False
    End If
  End If

  Rotary_Enc_A = temp_A
  Rotary_Enc_B = temp_B
End Function

'============================================================================
' decode number -> 7segment-LED
'============================================================================
Function led7seg_decode_number( num As Byte ) As Byte
  Select num
    Case 0  result = Digit_0
    Case 1  result = Digit_1
    Case 2  result = Digit_2
    Case 3  result = Digit_3
    Case 4  result = Digit_4
    Case 5  result = Digit_5
    Case 6  result = Digit_6
    Case 7  result = Digit_7
    Case 8  result = Digit_8
    Case 9  result = Digit_9
  EndSelect
End Function


'============================================================================
'  number -> led_val1 to led_val6
'============================================================================
Sub decode_number_setmode( dp_pos As Byte )
  Dim tmp(6) As Byte
  Dim i As Byte
  
  For i = 0 To 5
    tmp(i) = led7seg_decode_number( set_val(i) )
  Next
  
  tmp(dp_pos) = tmp(dp_pos) Or Digit_DP
  
  led_val1 = tmp(0)
  led_val2 = tmp(1)
  led_val3 = tmp(2)
  led_val4 = tmp(3)
  led_val5 = tmp(4)
  led_val6 = tmp(5)
End Sub

'============================================================================
'  display zero
'============================================================================
Sub display_zero()
  led_val1 = Digit_0
  led_val2 = Digit_N
  led_val3 = Digit_N
  led_val4 = Digit_N
  led_val5 = Digit_N
  led_val6 = Digit_N
End Sub


'*******************************************************************************
' Program main
'*******************************************************************************
Dim led_txt  As String(6)
Dim dp_pos    As Byte
Dim i As Byte
Dim multiple  As LongWord

PortB_pullups_disable  = true

PortB0_int_enable      = true
PortB0_int_rising_edge = true

ADconfig0 = 1  ' PortA Digital I/O
ADconfig1 = 1
ADconfig2 = 1 
ADconfig3 = 1

Comparator_mode2 = 1 ' compatator off
Comparator_mode1 = 1 ' compatator off
Comparator_mode0 = 1 ' compatator off


Input(PHOTO_REF_PORT)

Input(ENCODER_A)
Input(ENCODER_B)

Input(SW1)
Input(SW2)

Output(LED1)
Output(LED2)
Output(LED3)

Output(RELAY)

LED1 = LED_OFF
LED2 = LED_OFF
LED3 = LED_OFF

Output(LED_DEV6)
Output(LED_DEV5)
Output(LED_DEV4)
Output(LED_DEV3)
Output(LED_DEV2)
Output(LED_DEV1)
   
Output(LED_SEG_A)
Output(LED_SEG_B)
Output(LED_SEG_C)
Output(LED_SEG_D)
Output(LED_SEG_E)
Output(LED_SEG_F)
Output(LED_SEG_G)
Output(LED_SEG_DP)

LED_DEV6 = LED_OFF
LED_DEV5 = LED_OFF
LED_DEV4 = LED_OFF
LED_DEV3 = LED_OFF
LED_DEV2 = LED_OFF
LED_DEV1 = LED_OFF

led_val1 = Digit_8
led_val2 = Digit_8
led_val3 = Digit_8
led_val4 = Digit_8
led_val5 = Digit_8
led_val6 = Digit_8
led_pos  = 1  


'---- TMR0(8bitMODE) initialize --------------
Timer0_prescale_bit2 = 1   ' PreScaler 1:128
Timer0_prescale_bit1 = 1   ' PreScaler 1:128
Timer0_prescale_bit0 = 0   ' PreScaler 1:128
Timer0_prescale_NOT_assign = false
Timer0_mode_8bit           = true
Timer0_interrupt_enable    = true
Timer0_use_external_clock  = false
Timer0_enable              = true
Timer0_priority            = Low_priority

'---- TMR1(16bit) initialize -----------------
// Timer1_peripheral_int_enable = true
Timer1_interrupt_enable    = true 
Timer1_prescale_bit0       = 0
Timer1_prescale_bit1       = 0
Timer1_use_external_clock  = false
Timer1_enable              = true
Timer1_priority            = Low_priority

'---- interrpt -------------------------------
Enable(OnChangePORTB)
Enable(OnTimer)
Interrupt_priority    = true
Interrupt_HIGH_enable = true
Interrupt_LOW_enable  = true

Rotary_Enc_A = ENCODER_A
Rotary_Enc_B = ENCODER_B

SW1_status = SW_ENABLE 
SW2_status = SW_ENABLE 


RELAY = RELAY_OFF
dp_pos = 0
count_status = DISPLAY
rpm = 0
DelayMS(1000)
led_val1 = Digit_0
led_val2 = Digit_N
led_val3 = Digit_N
led_val4 = Digit_N
led_val5 = Digit_N
led_val6 = Digit_N

TMR1L_value    = 0
TMR1H_value    = 0
TMR1_OVF_value = 0
rotate_value   = 0
  
set_rpm = ReadLongWord(0)  
zero_display_counter = 0
start_flag   = false
display_flag = false
While( true )
  '---- Display MODE <-> SET MODE ----------
  If SW2 = SW_ON Then
    If SW2_status = SW_ENABLE Then
      '---- normal mode -> set mode ------
      If count_status = DISPLAY Then
        count_status = SET

        RELAY = RELAY_OFF
        LED3 = LED_OFF
        tmp_set_rpm = ReadLongWord(0)
        For i = 0 To 5
          set_val(i) = tmp_set_rpm Mod 10
          tmp_set_rpm = tmp_set_rpm / 10
        Next
 
        LED1 = LED_ON
      '---- set mode -> normal mode ------
      Else
        count_status = DISPLAY
        LED1 = LED_OFF
        
        tmp_set_rpm = 0
        multiple = 1
        For i = 0 To 5
          tmp_set_rpm = tmp_set_rpm + (LongWord(set_val(i)) * multiple)
          multiple = multiple * 10 
        Next
        If tmp_set_rpm <> 0 Then
          set_rpm = tmp_set_rpm
        Else
          set_rpm = 0
        End If
                
        WriteLongWord(0, set_rpm)
       
        display_zero()
        RELAY = RELAY_OFF
        LED3 = LED_OFF
      End If
      SW2_status = SW_DISABLE
    End If
  End If

  If SW2_status = SW_DISABLE Then  
    If SW2 = SW_OFF Then
      SW2_status = SW_ENABLE
    End If
  End If

  '---- SET MODE -------------------------
  If count_status = SET Then  
    
    '---- move decimal place ----------
    If SW1 = SW_ON Then
      If SW1_status = SW_ENABLE Then
        If dp_pos = 5 Then
          dp_pos = 0
        Else
          dp_pos = dp_pos + 1
        End If
        SW1_status = SW_DISABLE
      End If
    End If

    If SW1_status = SW_DISABLE Then  
      If SW1 = SW_OFF Then
        SW1_status = SW_ENABLE
      End If
    End If
    
    '---- rotary encoder --------------
    turn = rotary_encoder()
    Select turn
      Case CW 
        If set_val(dp_pos) < 9 Then
          set_val(dp_pos) = set_val(dp_pos) + 1
        End If
      Case CCW
        If set_val(dp_pos) > 0 Then
          set_val(dp_pos) = set_val(dp_pos) - 1
        End If
    EndSelect
             
    decode_number_setmode( dp_pos )         
  End If

  '---- Display MODE -----------------------
  If count_status = DISPLAY Then
  
    '---- rotary encoder --------------
    turn = rotary_encoder()
    Select turn
      Case CW 
        If set_val(dp_pos) < 9 Then
          set_val(dp_pos) = set_val(dp_pos) + 1
        End If
      Case CCW
        If set_val(dp_pos) > 0 Then
          set_val(dp_pos) = set_val(dp_pos) - 1
        End If
    EndSelect
  
  
    If zero_display_counter >= ZERO_CLEAR_TIME Then
      display_zero()
      RELAY = RELAY_OFF
      LED3 = LED_OFF
      zero_display_counter = 0
      start_flag = false
    End If
    If display_flag Then
      tempLWB.ByteVal(0) = TMR1L_value
      tempLWB.ByteVal(1) = TMR1H_value
      tempLWB.WordVal(1) = TMR1_OVF_value
      tmr1_count = tempLWB.LongwordVal

      count_div  = CONST_MIN / tmr1_count
      count_tmp1 = count_div * rotate_value
      count_tmp2 = CONST_MIN Mod tmr1_count
      count_tmp2 = count_tmp2 * rotate_value
      count_tmp2 = count_tmp2 / tmr1_count
      rpm = count_tmp1 + count_tmp2

      If rpm >= set_rpm Then
        RELAY = RELAY_ON
        LED3 = LED_ON
      Else
        RELAY = RELAY_OFF
        LED3 = LED_OFF
      End If

      led_txt = DecToStr( rpm, 6, " " )
      led_val1 = led7seg_decode( led_txt(5) )
      led_val2 = led7seg_decode( led_txt(4) )
      led_val3 = led7seg_decode( led_txt(3) )
      led_val4 = led7seg_decode( led_txt(2) )
      led_val5 = led7seg_decode( led_txt(1) )
      led_val6 = led7seg_decode( led_txt(0) )

      display_flag = false
    End If
  End If
Wend


'--------- E N D  O F  P R O G R A M ------------------------------------------- 
 
